﻿using UnityEngine;
using System.Collections;
using Zenject;
using System;
using UnityEngine.SceneManagement;

public class DefaultGroundMovement : Movement
{
	public override void Tick()
	{
		if (SceneController.instance.isFading)
			return;
		if (!pawn.isDead) 
		{
			GroundCheck ();
			Jump ();
		}
	}

	public override void Kill()
	{
		if (pawn.isDead)
			return;
		pawn.meshRoot.GetComponent<Animation>().enabled = false;
		pawn.characterAnimator.enabled = false;
		pawn.isDead = true;
		foreach (var item in pawn.parentRoot.gameObject.GetComponentsInChildren<Rigidbody>()) 
		{
			item.isKinematic = false;
		}	
		pawn.parentRoot.gameObject.GetComponent<Rigidbody> ().isKinematic = true;
	}	

	public override void Revive()
	{
		if (!pawn.isDead)
			return;
		pawn.meshRoot.GetComponent<Animation>().enabled = true;
		pawn.characterAnimator.enabled = true;
		pawn.isDead = false;
		foreach (var item in pawn.parentRoot.gameObject.GetComponentsInChildren<Rigidbody>()) 
		{
			item.isKinematic = true;
		}	
		pawn.parentRoot.gameObject.GetComponent<Rigidbody> ().isKinematic = false;
		pawn.meshRoot.GetComponentInChildren<Rigidbody> ().gameObject.transform.position = Vector3.zero;
	}

	public override void FixedTick()
	{		
		if (!SceneController.instance.isFading) 
		{
			if (!pawn.isDead) 
			{
				//GroundCheck ();
				Move ();
				Rotate ();
				ClampAngle ();
			}
		} 
		else
		{
			SceneTransition ();
		}
        UpateMovement();
    }

	public override void OnCollisionStay(Collision other)
	{
		if (SceneController.instance.isFading)
			return;
		settings.Collision = true;
		if (other.gameObject.CompareTag("Platform") && settings.Grounded)
		{
			pawn.parentRoot.gameObject.transform.parent = other.transform;
		}
	}	

	public override void OnCollisionExit(Collision other)
	{
		if (SceneController.instance.isFading)
			return;
		settings.Collision = false;
		pawn.parentRoot.gameObject.transform.parent = null;
	}

	public override void Jump()
	{
		if (settings.Grounded && settings.Collision) 
		{
			if (Input.GetKeyDown(KeyCode.Space) && (pawn.characterAnimator.GetCurrentAnimatorStateInfo(0).IsName("Idle") || pawn.characterAnimator.GetCurrentAnimatorStateInfo(0).IsName("Run"))) 
			{
				pawn.characterAnimator.SetTrigger("jump");
				//settings.Grounded = false;
				//settings.jumpedground = true;
				settings.rb.AddForce (Vector3.up * settings.jumpForce, ForceMode.Impulse);
			}
		}
	}

    public override void Move()
    {
		if (pawn.parentRoot.position.y < -30 && !SceneController.instance.isFading)
		{
			SceneController.instance.StartCoroutine(SceneController.instance.FadeAndLoadScene(SceneManager.GetActiveScene().name, "initialPosition"));
		}
		if (SceneController.instance.isFading) 
		{
			pawn.parentRoot.transform.position = new Vector3 (pawn.parentRoot.transform.position.x, settings.GroundHit.y,pawn.parentRoot.transform.position.z);	
		}
		if (settings.Grounded) 
		{
			if(settings.rb.velocity.z > 15f)
				settings.rb.velocity= new Vector3(settings.rb.velocity.x, settings.rb.velocity.y, 15f);

			if(settings.rb.velocity.z < -15f)
				settings.rb.velocity= new Vector3(settings.rb.velocity.x, settings.rb.velocity.y, -15f);
		}
		else {
			if(settings.rb.velocity.z > 13f)
				settings.rb.velocity= new Vector3(settings.rb.velocity.x, settings.rb.velocity.y, 13f);

			if(settings.rb.velocity.z < -13f)
				settings.rb.velocity= new Vector3(settings.rb.velocity.x, settings.rb.velocity.y, -13f);
		}
		settings.rb.AddForce (inputState.WorldMovementInput * settings.maxVelocity * Time.fixedDeltaTime, ForceMode.Acceleration);


		
		/*
		//pawn.parentRoot.position += (pawn.meshRoot.transform.forward * settings.currentVelocity * settings.velocityMultiplier * settings.angleMultiplier * Time.deltaTime);
		//pawn.parentRoot.position = new Vector3(0, pawn.parentRoot.position.y, pawn.parentRoot.position.z);
		settings.currentVelocity += settings.acceleration * inputState.WorldMovementInput.sqrMagnitude * Time.deltaTime;
		settings.currentVelocity -= settings.drag * Time.deltaTime;
		if (settings.currentVelocity > settings.maxVelocity)
			settings.currentVelocity = settings.maxVelocity;
		if (settings.currentVelocity < 0)
			settings.currentVelocity = 0;
		*/
    }

    public override void Rotate()
    {
        if (settings.Grounded)
        {

        }
        else
        {
            pawn.meshRoot.transform.rotation = Quaternion.Lerp(pawn.meshRoot.transform.rotation, Quaternion.LookRotation(settings.lookDirection, Vector3.up), settings.turnScale * Time.deltaTime);
        }

        pawn.meshRoot.transform.rotation = Quaternion.Slerp(pawn.meshRoot.transform.rotation, settings.groundQuaternion, settings.turnScale * Time.deltaTime);

        /**KNOCKBACK

		//bool invertKnockBack = false;
		//groundMoveDirection = Vector3.ProjectOnPlane(movementInput, groundNormal);
		//Quaternion groundMoveQuaternion = Quaternion.LookRotation(groundMoveDirection, groundNormal);


		//if (groundMoveQuaternion.eulerAngles.x < 180)
		//{
		//    groundMoveQuaternion.eulerAngles = new Vector3(groundMoveQuaternion.eulerAngles.x, groundMoveQuaternion.eulerAngles.y, groundMoveQuaternion.eulerAngles.z);
		//}
		//else if (groundMoveQuaternion.eulerAngles.x > 180)
		//{
		//    invertKnockBack = true;
		//    groundMoveQuaternion.eulerAngles = new Vector3(360 - groundMoveQuaternion.eulerAngles.x, groundMoveQuaternion.eulerAngles.y, groundMoveQuaternion.eulerAngles.z);
		//}

		//if (groundMoveQuaternion.eulerAngles.z < 180)
		//{
		//    groundMoveQuaternion.eulerAngles = new Vector3(groundMoveQuaternion.eulerAngles.x, groundMoveQuaternion.eulerAngles.y, groundMoveQuaternion.eulerAngles.z);
		//}
		//else if (groundMoveQuaternion.eulerAngles.z > 180)
		//{
		//    //invertKnockBack = true;
		//    groundMoveQuaternion.eulerAngles = new Vector3(groundMoveQuaternion.eulerAngles.x, groundMoveQuaternion.eulerAngles.y, 360 - groundMoveQuaternion.eulerAngles.z);
		//}

		//if (groundMoveQuaternion.eulerAngles.x > stopAngle)
		//{
		//    groundMoveQuaternion.eulerAngles = new Vector3(stopAngle, groundMoveQuaternion.eulerAngles.y, groundMoveQuaternion.eulerAngles.z);
		//}
		//if (groundMoveQuaternion.eulerAngles.z > stopAngle)
		//{
		//    groundMoveQuaternion.eulerAngles = new Vector3(groundMoveQuaternion.eulerAngles.x, groundMoveQuaternion.eulerAngles.y, stopAngle);
		//}

		//Vector3 inputMoveGroundDirection = groundMoveQuaternion.eulerAngles / stopAngle;
		//inputMoveGroundDirection = new Vector3(inputMoveGroundDirection.x, 0, inputMoveGroundDirection.z);
		//groundDirectionMagnitude = inputMoveGroundDirection.sqrMagnitude;
		//angleMultiplier = 1;

		//if ((groundDirectionMagnitude >= .4f || groundMoveQuaternion.x == stopAngle || groundMoveQuaternion.z == stopAngle) && freezeGroundInput == Vector3.zero && movementInput != Vector3.zero)
		//{
		//    freezeGroundInput = movementInput;
		//    if (!invertKnockBack)
		//    {
		//        angleMultiplier = groundDirectionMagnitude / 1;
		//    }
		//    else
		//    {
		//        angleMultiplier = 0;
		//    }
		//}
		//else if (freezeGroundInput != Vector3.zero && lastFrameInput != Vector3.zero)
		//{
		//    freezeGroundInput = movementInput;
		//    if (!invertKnockBack)
		//    {
		//        knockBackForce = freezeGroundInput * angleKnockBackForce;
		//    }
		//    else
		//    {
		//        knockBackForce = (freezeGroundInput * -1) * angleKnockBackForce;
		//    }
		//    freezeGroundInput = Vector3.zero;
		//}
        */
    }

    public override void UpateMovement()
	{
		pawn.characterAnimator.SetBool("grounded", settings.Grounded);
        if (settings.Grounded)
		{
			pawn.characterAnimator.SetFloat("velocity", settings.rb.velocity.z * inputState.WorldMovementInput.z);
        }
        else
        {
            pawn.characterAnimator.SetFloat("velocity", 0, .1f, Time.deltaTime);
        }
		if (SceneController.instance.isFading) 
		{
			pawn.characterAnimator.SetFloat("velocity", 0);
		}

        settings.lastFrameVelocity = settings.currentVelocity;
    }

	public override IEnumerator WaitForInteraction ()
	{
		settings.handleInput = false;

		yield return settings.inputHoldWait;

		settings.handleInput = true;
	}
}