﻿using InControl;
using UnityEngine;


public class PlayerInputActions : PlayerActionSet
{
	
	public PlayerAction WorldContextualAction;
	public PlayerAction WorldOpenMenu;
	public PlayerAction WorldOpenPause;
	public PlayerAction WorldOpenSkit;
	public PlayerAction MenuBack;
	public PlayerAction MenuSelect;
	public PlayerAction BattleSelect;
	public PlayerAction BattleConfirm;
	public PlayerAction BattleBack;
	public PlayerAction Left;
	public PlayerAction Right;
	public PlayerAction Up;
	public PlayerAction Down;
	public PlayerAction Jump;
	public PlayerTwoAxisAction Move;

	public PlayerInputActions()
	{
		WorldContextualAction = CreatePlayerAction( "World Contextual Action" );
		WorldOpenMenu = CreatePlayerAction( "World Open Menu" );
		WorldOpenPause = CreatePlayerAction( "World Open Pause" );
		WorldOpenSkit = CreatePlayerAction( "World Open Skit" );
		MenuBack = CreatePlayerAction( "Menu Back" );
		MenuSelect = CreatePlayerAction( "Menu Select" );
		BattleSelect = CreatePlayerAction( "Battle Select" );
		BattleConfirm = CreatePlayerAction( "Battle Confirm" );
		BattleBack = CreatePlayerAction( "Battle Back" );
		Left = CreatePlayerAction( "Move Left" );
		Right = CreatePlayerAction( "Move Right" );
		Up = CreatePlayerAction( "Move Up" );
		Down = CreatePlayerAction( "Move Down" );
		Move = CreateTwoAxisPlayerAction( Left, Right, Down, Up );
		Jump = CreatePlayerAction( "jump" );
	}


	public static PlayerInputActions CreateWithDefaultBindings()
	{
		var playerActions = new PlayerInputActions();

		playerActions.WorldContextualAction.AddDefaultBinding( Key.Shift );
		playerActions.WorldContextualAction.AddDefaultBinding( InputControlType.Action1 );

		playerActions.WorldOpenMenu.AddDefaultBinding( Key.Space );
		playerActions.WorldOpenMenu.AddDefaultBinding( InputControlType.Action4 );

		playerActions.WorldOpenPause.AddDefaultBinding( Key.Return );
		playerActions.WorldOpenPause.AddDefaultBinding( InputControlType.Start );

		playerActions.WorldOpenSkit.AddDefaultBinding( Key.Q );
		playerActions.WorldOpenSkit.AddDefaultBinding( InputControlType.Action3 );

		playerActions.MenuBack.AddDefaultBinding( Key.Backspace );
		playerActions.MenuBack.AddDefaultBinding( InputControlType.Action2 );

		playerActions.MenuSelect.AddDefaultBinding( Key.Shift );
		playerActions.MenuSelect.AddDefaultBinding( InputControlType.Action1 );

		playerActions.BattleSelect.AddDefaultBinding( Key.Shift );
		playerActions.BattleSelect.AddDefaultBinding( InputControlType.Action1 );

		playerActions.BattleConfirm.AddDefaultBinding( Key.Return );
		playerActions.BattleConfirm.AddDefaultBinding( InputControlType.Action4 );

		playerActions.BattleBack.AddDefaultBinding( Key.Backspace );
		playerActions.BattleBack.AddDefaultBinding( InputControlType.Action2 );

		//playerActions.Up.AddDefaultBinding( Key.UpArrow );
		//playerActions.Down.AddDefaultBinding( Key.DownArrow );
		playerActions.Left.AddDefaultBinding( Key.LeftArrow );
		playerActions.Right.AddDefaultBinding( Key.RightArrow );

		//playerActions.Up.AddDefaultBinding( Key.W );
		//playerActions.Down.AddDefaultBinding( Key.S );
		playerActions.Left.AddDefaultBinding( Key.A );
		playerActions.Right.AddDefaultBinding( Key.D );

		playerActions.Left.AddDefaultBinding( InputControlType.LeftStickLeft );
		playerActions.Right.AddDefaultBinding( InputControlType.LeftStickRight );
		//playerActions.Up.AddDefaultBinding( InputControlType.LeftStickUp );
		//playerActions.Down.AddDefaultBinding( InputControlType.LeftStickDown );

		playerActions.Left.AddDefaultBinding( InputControlType.DPadLeft );
		playerActions.Right.AddDefaultBinding( InputControlType.DPadRight );
		//playerActions.Up.AddDefaultBinding( InputControlType.DPadUp );
		//playerActions.Down.AddDefaultBinding( InputControlType.DPadDown );

		playerActions.Jump.AddDefaultBinding( Key.Space );
		playerActions.Jump.AddDefaultBinding( InputControlType.Action1 );

		playerActions.ListenOptions.IncludeUnknownControllers = true;
		playerActions.ListenOptions.MaxAllowedBindings = 4;
		//playerActions.ListenOptions.MaxAllowedBindingsPerType = 1;
		//playerActions.ListenOptions.AllowDuplicateBindingsPerSet = true;
		//playerActions.ListenOptions.UnsetDuplicateBindingsOnSet = true;
		//playerActions.ListenOptions.IncludeMouseButtons = true;
		//playerActions.ListenOptions.IncludeModifiersAsFirstClassKeys = true;

		playerActions.ListenOptions.OnBindingFound = ( action, binding ) => {
			if (binding == new KeyBindingSource( Key.Escape ))
			{
				action.StopListeningForBinding();
				return false;
			}
			return true;
		};

		playerActions.ListenOptions.OnBindingAdded += ( action, binding ) => {
			Debug.Log( "Binding added... " + binding.DeviceName + ": " + binding.Name );
		};

		playerActions.ListenOptions.OnBindingRejected += ( action, binding, reason ) => {
			Debug.Log( "Binding rejected... " + reason );
		};

		return playerActions;
	}
}

