﻿using UnityEngine;
using System;
using UnityEngine.AI;

public enum DefaultCompanionMoveTypes { None, Default }

public class CompanionInstaller : PawnInstaller
{
    [SerializeField]
    public Settings settings = null;

    public override void InstallBindings()
    {
        Install();
        BindInstances(settings);
    }

    public override void Install()
    {
        InstallPawn(settings);
        InstallInput(settings);
        InstallMovement(settings);
    }

    public virtual void InstallMovement(Settings settings)
    {
        base.InstallMovement(settings);
        switch (settings.moveType)
        {
            case DefaultPlayerMoveTypes.None:
                break;
            case DefaultPlayerMoveTypes.Default:
                Container.BindAllInterfacesAndSelf<IMovement>().To<DefaultCompanionGroundFollowMovement>().AsSingle();
                break;
            default:
                break;
        }
    }


    public virtual void InstallInput(Settings settings)
    {
        base.InstallInput(settings);
    }

    public virtual void CopyPawnReferences(Settings settings)
    {
        base.CopyPawnReferences(settings);
        settings.controller.movement.settings.agent = settings.agent;
		if(settings.controller.movement.settings.followPlayer)
			settings.controller.movement.settings.followTarget = GameObject.FindWithTag ("Player").transform.GetComponent<FieldInstaller>().settings.controller.pawn.settings.meshRoot.transform;
		else 
		{
			foreach (var item in GameObject.FindGameObjectsWithTag ("Companion")) 
			{
				if (item != gameObject) 
				{
					settings.controller.movement.settings.followTarget = item.transform;
				}
			}
		}
    }

    public virtual void InstallPawn(Settings settings)
    {
        CopyPawnReferences(settings);
        base.InstallPawn(settings);
    }

    public virtual void BindInstances(Settings settings)
    {
        base.BindInstances(settings);
    }

    [Serializable]
    new public class Settings : PawnInstaller.Settings
    {
        public DefaultPlayerMoveTypes moveType;
        public NavMeshAgent agent;
    }
}