﻿using System;
using UnityEngine;
using System.Collections;
using UnityEngine.SceneManagement;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

public class SceneController : MonoBehaviour
{
	public event Action BeforeSceneUnload;          // Event delegate that is called just before a scene is unloaded.
	public event Action AfterSceneLoad;             // Event delegate that is called just after a scene is loaded.

	public static SceneController instance;
	public CanvasGroup faderCanvasGroup;
	public float fadeDuration = 1f;
	public string startingSceneName;
	public string initialStartingPositionName;
	public SaveReliableData playerSaveData;
	public bool isFading = true;

    public string[] scenes;
    void Awake()
    {
        startingSceneName = scenes[PlayerPrefs.GetInt("selectedLevel")];
    }

	private IEnumerator Start ()
	{
		instance = this;
		faderCanvasGroup.alpha = 1f;

		if (initialStartingPositionName != "")
			playerSaveData.SaveString (DATAStrings.position, initialStartingPositionName);

		playerSaveData.LoadString (DATAStrings.position, ref initialStartingPositionName);	

		if (startingSceneName != "")
			playerSaveData.SaveString (DATAStrings.level, startingSceneName);

		playerSaveData.LoadString (DATAStrings.level, ref startingSceneName);

		yield return StartCoroutine (LoadSceneAndSetActive(startingSceneName, initialStartingPositionName));

		//string newPos = "";
		//Load(out newPos);
		//playerSaveData.Save (PlayerMovement.startingPositionKey, newPos);
		//playerSaveData.stringKeyValuePairLists.TryGetValue(PlayerMovement.startingPositionKey, ref newPos);
		//yield return StartCoroutine (LoadSceneAndSetActive(startingSceneName, newPos));
		StartCoroutine (Fade (0f));
		//Debug.Log (playerSaveData.Load(PlayerMovement.startingPositionKey));
	}

	public IEnumerator FadeAndLoadScene(string sceneName)
	{
		if (!isFading)
		{
			yield return StartCoroutine (Fade (1f));
			yield return SceneManager.UnloadSceneAsync (SceneManager.GetActiveScene().buildIndex);
			yield return StartCoroutine (LoadSceneAndSetActive (sceneName));
			yield return StartCoroutine (Fade (0f));
		}
	}

	public IEnumerator FadeAndLoadScene(string sceneName, string newPosition)
	{
		playerSaveData.SaveString (DATAStrings.position.ToString(), newPosition);
		if (!isFading)
		{		
			yield return new WaitForSeconds (1.5f);
			yield return StartCoroutine (Fade (1f));
			GameObject.FindGameObjectWithTag("Player").transform.SetParent(gameObject.transform);
			yield return SceneManager.UnloadSceneAsync (SceneManager.GetActiveScene().buildIndex);
			yield return StartCoroutine (LoadSceneAndSetActive (sceneName));
			SetNewPlayerPosition(GameObject.FindGameObjectWithTag("Player"), newPosition);
			yield return StartCoroutine (Fade (0f));
		}
	}

	private IEnumerator FadeAndSwitchScenes (string sceneName)
	{
		yield return StartCoroutine (Fade (1f));
		yield return SceneManager.UnloadSceneAsync (SceneManager.GetActiveScene().buildIndex);
		yield return StartCoroutine (LoadSceneAndSetActive (sceneName));
		yield return StartCoroutine (Fade (0f));
	}

	private IEnumerator LoadSceneAndSetActive (string sceneName, string newPosition)
	{
		yield return SceneManager.LoadSceneAsync(sceneName, LoadSceneMode.Additive);
		Scene newlyLoadedScene = SceneManager.GetSceneAt(SceneManager.sceneCount - 1);
		SetNewPlayerPosition(GameObject.FindGameObjectWithTag("Player"), newPosition);
		SceneManager.SetActiveScene (newlyLoadedScene);
	}	

	private IEnumerator LoadSceneAndSetActive (string sceneName)
	{
		yield return SceneManager.LoadSceneAsync(sceneName, LoadSceneMode.Additive);
		Scene newlyLoadedScene = SceneManager.GetSceneAt(SceneManager.sceneCount - 1);
		SceneManager.SetActiveScene (newlyLoadedScene);
	}

	private IEnumerator Fade (float finalAlpha)
	{
		isFading = true;
		faderCanvasGroup.blocksRaycasts = true;
		float fadeSpeed = Mathf.Abs (faderCanvasGroup.alpha - finalAlpha) / fadeDuration;
		while (!Mathf.Approximately (faderCanvasGroup.alpha, finalAlpha))
		{
			faderCanvasGroup.alpha = Mathf.MoveTowards (faderCanvasGroup.alpha, finalAlpha,
				fadeSpeed * Time.deltaTime);
			yield return null;
		}
		isFading = false;
		faderCanvasGroup.blocksRaycasts = false;
	}

	public void SetNewPlayerPosition(GameObject player, string newPlayerPosition)
	{
		GameObject newPosition = GameObject.Find (newPlayerPosition);
		player.transform.position = newPosition.transform.position;
		player.transform.rotation = newPosition.transform.rotation;
	}

	void OnEnable() {
		SceneManager.sceneLoaded += OnSceneLoaded;
	}

	void OnDisable() {
		SceneManager.sceneLoaded -= OnSceneLoaded;
	}

	private void OnSceneLoaded(Scene scene, LoadSceneMode mode) 
	{
		PlayerFacade player = GameObject.FindGameObjectWithTag("Player").GetComponent<PlayerFacade> ();
		player.movement.Revive ();
	}
}